'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (_ref) {
	var t = _ref.types;

	var classVisitor = function classVisitor(path) {
		if (!path.node.decorators) {
			return;
		}

		var decoratorNames = path.node.decorators.map(function (decorator) {
			return decorator.expression.callee.name;
		});
		var hasDecorator = decoratorNames.some(function (name) {
			return DECORATORS.includes(name);
		});

		if (!hasDecorator) {
			return;
		}

		var ctorNode = getCtorNode(path.node);

		if (!ctorNode) {
			return;
		}

		var paramNames = getMethodParamNames(ctorNode);

		var injectExp = t.expressionStatement(t.assignmentExpression('=', t.memberExpression(t.identifier(path.node.id.name), t.identifier('$inject')), t.arrayExpression(paramNames.map(function (name) {
			return t.stringLiteral(name);
		}))));

		if (path.parentPath.type === 'ExportDefaultDeclaration') {
			path.parentPath.insertAfter(injectExp);
		} else {
			path.insertAfter(injectExp);
		}
	};

	return {
		visitor: {
			ClassDeclaration: classVisitor
		}
	};
};

var DECORATORS = ['Component', 'Directive', 'Controller'];

function getCtorNode(node) {
	return node.body.body.find(function (def) {
		return def.kind === 'constructor';
	});
}

function getMethodParamNames(node) {
	return node.params.map(function (param) {
		return param.name;
	});
}

/**
 * Babel plugin to automatically annotate functions for Angular dependency injection.
 * @param {Object} babel The current babel object.
 * @param {Object} babel.types babel-types
 * @return {Object} Babel visitor.
 */